==> open-xchange-guard_2.4.2.orig.tar.bz2: open-xchange-guard-2.4.2/com.openexchange.guard.pgpcore.services.impl/src/com/openexchange/guard/pgpcore/services/impl/DBKeyRetrievalStrategy.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.pgpcore.services.impl;

import java.util.Iterator;
import org.bouncycastle.openpgp.PGPException;
import org.bouncycastle.openpgp.PGPPrivateKey;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPPublicKeyRing;
import org.bouncycastle.openpgp.PGPSecretKey;
import org.bouncycastle.openpgp.PGPSecretKeyRing;
import org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor;
import org.bouncycastle.openpgp.operator.bc.BcPBESecretKeyDecryptorBuilder;
import org.bouncycastle.openpgp.operator.bc.BcPGPDigestCalculatorProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.exception.OXException;
import com.openexchange.guard.common.util.CipherUtil;
import com.openexchange.guard.email.storage.ogEmail.Email;
import com.openexchange.guard.email.storage.ogEmail.EmailStorage;
import com.openexchange.guard.keys.dao.GuardKeys;
import com.openexchange.guard.keys.dao.PGPKeys;
import com.openexchange.guard.keys.storage.KeyTableStorage;
import com.openexchange.guard.keys.storage.PGPKeysStorage;
import com.openexchange.guard.pgpcore.commons.PGPKeyRetrievalStrategy;
import com.openexchange.guard.pgpcore.services.exceptions.PGPCoreServicesExceptionCodes;

/**
 * {@link DBKeyRetrievalStrategy}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.4.2
 */
public class DBKeyRetrievalStrategy implements PGPKeyRetrievalStrategy {

    private static final Logger logger = LoggerFactory.getLogger(DBKeyRetrievalStrategy.class);
    private final PGPKeysStorage pgpKeysStorage;
    private final EmailStorage emailStorage;
    private final KeyTableStorage keyTableStorage;

    /**
     * Initializes a new {@link DBKeyRetrievalStrategy}.
     * 
     * @param pgpKeysStorage The PGPKeysStorage service
     * @param emailStorage The EmailStorage service
     * @param keyTableStorage The KeyTableStorage service
     */
    public DBKeyRetrievalStrategy(PGPKeysStorage pgpKeysStorage, EmailStorage emailStorage, KeyTableStorage keyTableStorage) {
        this.pgpKeysStorage = pgpKeysStorage;
        this.emailStorage = emailStorage;
        this.keyTableStorage = keyTableStorage;
    }
    
    private boolean hasUserId(PGPSecretKey key, String userIdentity) {
        Iterator<String> userIDs = key.getUserIDs();
        while(userIDs.hasNext()) {
            String id = userIDs.next();
            if(id.contains(userIdentity)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Retrieves Guard Key information for a given key id
     * 
     * @param keyId The id of the key
     * @return The GuardKey for the given key id, or null if not key was found
     * @throws OXException
     */
    private GuardKeys getGuardKeyById(long keyId) throws OXException {

        //This mapping will provide the email address for the given key id
        PGPKeys keyMapping = pgpKeysStorage.getById(keyId);
        if (keyMapping != null) {
            //This mapping will provide the user- and context-id
            Email emailMapping = emailStorage.getByEmail(keyMapping.getEmail());
            if (emailMapping != null) {
                //Getting the keys for the user-id, cid, and keyId
                GuardKeys guardKeys = keyTableStorage.getKeyForUserById(emailMapping.getUserId(), emailMapping.getContextId(), emailMapping.getShardingId(), Long.toString(keyMapping.getKeyId()));
                if (guardKeys != null) {
                    return guardKeys;
                } 
                else {
                    logger.debug("Unable to get ogKeyTable mapping for key id %s.", Long.toString(keyId));
                }
            }
            else {
                logger.debug("Unable to get ogEmail mapping for key id %s.", Long.toString(keyId));
            }
        } else {
            logger.debug("Unable to get PGPKeys mapping for key id %s.", Long.toString(keyId));
        }
        return null;
    }
    
    /**
     * Extracts a private key from a PGPSecretKey object
     * 
     * @param secretKey The PGPSecretKeyObject
     * @param password The password 
     * @return The decoded private key
     * @throws PGPException
     */
    private PGPPrivateKey decodePrivateKey(PGPSecretKey secretKey, char[] password) throws Exception {        
        try {
            PBESecretKeyDecryptor extractor = new BcPBESecretKeyDecryptorBuilder(new BcPGPDigestCalculatorProvider()).build(password);
            return secretKey.extractPrivateKey(extractor);
        } catch (PGPException e) {
            throw PGPCoreServicesExceptionCodes.BAD_PASSWORD.create();
        }
    }
    
    /**
     * @param password
     * @param salt
     * @return
     */
    private char[] hashPassword(char[] password, String salt) {
        return CipherUtil.getSHA(new String(password), salt).toCharArray();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.guard.pgpcore.PGPKeyRetrievalStrategy#getSecretKey(long)
     */
    @Override
    public PGPPrivateKey getSecretKey(long keyId, String userIdentity, char[] password) throws Exception {
        GuardKeys guardKeys = getGuardKeyById(keyId);
        if (guardKeys != null) {
            PGPSecretKeyRing pgpSecretKeyRing = guardKeys.getPGPSecretKeyRing();            
            PGPSecretKey secretKey = pgpSecretKeyRing.getSecretKey(keyId);
            if (secretKey != null) {
                if(userIdentity == null) {
                    //Do not check for userIdentity
                    return decodePrivateKey(secretKey, hashPassword(password,guardKeys.getSalt()));
                }
                else {
                    //Check if we find the userIdentity within the key, or master-key
                    if(hasUserId(secretKey,userIdentity) || hasUserId(pgpSecretKeyRing.getSecretKey()/*masterKey*/,userIdentity)) {
                        return decodePrivateKey(secretKey, hashPassword(password,guardKeys.getSalt()));
                    }
                }
            }
            else {
                logger.debug("Unable to extract PGPSecretKey with key id %s from PGPSecretKeyRing.", Long.toString(keyId));
            }
        }
        logger.debug("Unable to find PGPSecretKey for key id %s.", Long.toString(keyId));
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.guard.pgpcore.PGPKeyRetrievalStrategy#getPublicKey(long)
     */
    @Override
    public PGPPublicKey getPublicKey(long keyId) throws Exception {
        GuardKeys guardKeys = getGuardKeyById(keyId);
        if (guardKeys != null) {
            PGPPublicKeyRing publicKeyRing = guardKeys.getPublicKeyRing();
            PGPPublicKey publicKey = publicKeyRing.getPublicKey(keyId);
            if (publicKey != null) {
                return publicKey;
            }
            else {
                logger.debug("Unable to extract PGPPublicKey with key id %s from PGPPublicKeyRing.", Long.toString(keyId));
            }
        }
        logger.debug("Unable to find PGPPublicKey for keyId %s.", Long.toString(keyId));
        return null;
    }
}

==> open-xchange-guard_2.6.0.orig.tar.bz2: open-xchange-guard-2.6.0/com.openexchange.guard.pgpcore.services.impl/src/com/openexchange/guard/pgpcore/services/impl/DBKeyRetrievalStrategy.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.pgpcore.services.impl;

import java.util.Iterator;
import org.bouncycastle.openpgp.PGPException;
import org.bouncycastle.openpgp.PGPPrivateKey;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPPublicKeyRing;
import org.bouncycastle.openpgp.PGPSecretKey;
import org.bouncycastle.openpgp.PGPSecretKeyRing;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.exception.OXException;
import com.openexchange.guard.common.util.CipherUtil;
import com.openexchange.guard.exceptions.GuardAuthExceptionCodes;
import com.openexchange.guard.keymanagement.commons.GuardKeys;
import com.openexchange.guard.keymanagement.commons.util.PGPUtil;
import com.openexchange.guard.keymanagement.services.GuardKeyService;
import com.openexchange.guard.pgpcore.commons.PGPKeyRetrievalStrategy;

/**
 * {@link DBKeyRetrievalStrategy}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.4.2
 */
public class DBKeyRetrievalStrategy implements PGPKeyRetrievalStrategy {

    private static final Logger logger = LoggerFactory.getLogger(DBKeyRetrievalStrategy.class);
    private final GuardKeyService guardKeyService;

    /**
     * Initializes a new {@link DBKeyRetrievalStrategy}.
     *
     * @param pgpKeysStorage The PGPKeysStorage service
     * @param emailStorage The EmailStorage service
     * @param keyTableStorage The KeyTableStorage service
     */
    public DBKeyRetrievalStrategy(GuardKeyService guardKeyService) {
        this.guardKeyService = guardKeyService;
    }

    private boolean hasUserId(PGPSecretKey key, String userIdentity) {
        Iterator<String> userIDs = key.getUserIDs();
        while(userIDs.hasNext()) {
            String id = userIDs.next();
            if(id.contains(userIdentity)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Retrieves Guard Key information for a given key id
     *
     * @param keyId The id of the key
     * @return The GuardKey for the given key id, or null if not key was found
     * @throws OXException
     */
    private GuardKeys getGuardKeyById(long keyId) throws OXException {
        return guardKeyService.getKeys(keyId);
    }

    /**
     * Extracts a private key from a PGPSecretKey object
     *
     * @param secretKey The PGPSecretKeyObject
     * @param password The password
     * @return The decoded private key
     * @throws PGPException
     */
    private PGPPrivateKey decodePrivateKey(PGPSecretKey secretKey, char[] password) throws Exception {
        try {
            return PGPUtil.decodePrivate(secretKey, new String(password));
        } catch (PGPException e) {
            throw GuardAuthExceptionCodes.BAD_PASSWORD.create();
        }
    }

    /**
     * @param password
     * @param salt
     * @return
     */
    private char[] hashPassword(char[] password, String salt) {
        return CipherUtil.getSHA(new String(password), salt).toCharArray();
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.pgpcore.PGPKeyRetrievalStrategy#getSecretKey(long)
     */
    @Override
    public PGPPrivateKey getSecretKey(long keyId, String userIdentity, char[] password) throws Exception {
        GuardKeys guardKeys = getGuardKeyById(keyId);
        if (guardKeys != null) {
            PGPSecretKeyRing pgpSecretKeyRing = guardKeys.getPGPSecretKeyRing();
            PGPSecretKey secretKey = pgpSecretKeyRing.getSecretKey(keyId);
            if (secretKey != null) {
                if(userIdentity == null) {
                    //Do not check for userIdentity
                    return decodePrivateKey(secretKey, hashPassword(password,guardKeys.getSalt()));
                }
                else {
                    //Check if we find the userIdentity within the key, or master-key
                    if(hasUserId(secretKey,userIdentity) || hasUserId(pgpSecretKeyRing.getSecretKey()/*masterKey*/,userIdentity)) {
                        return decodePrivateKey(secretKey, hashPassword(password,guardKeys.getSalt()));
                    }
                }
            }
            else {
                logger.debug("Unable to extract PGPSecretKey with key id %s from PGPSecretKeyRing.", Long.toString(keyId));
            }
        }
        logger.debug("Unable to find PGPSecretKey for key id %s.", Long.toString(keyId));
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.pgpcore.PGPKeyRetrievalStrategy#getPublicKey(long)
     */
    @Override
    public PGPPublicKey getPublicKey(long keyId) throws Exception {
        GuardKeys guardKeys = getGuardKeyById(keyId);
        if (guardKeys != null) {
            PGPPublicKeyRing publicKeyRing = guardKeys.getPGPPublicKeyRing();
            PGPPublicKey publicKey = publicKeyRing.getPublicKey(keyId);
            if (publicKey != null) {
                return publicKey;
            }
            else {
                logger.debug("Unable to extract PGPPublicKey with key id %s from PGPPublicKeyRing.", Long.toString(keyId));
            }
        }
        logger.debug("Unable to find PGPPublicKey for keyId %s.", Long.toString(keyId));
        return null;
    }
}
